@@ -86,29 +86,64 @@ sub initialize {
 	$self->{options}{'tagsonly'}=1;
 	$self->{options}{'tags'}.='
 		w<abstract>
+		<author>
+		<b>
+		<brite>
+		<c>
 		<codenote>
 		<comment>
+		<const>
 		<date>
-		<impo>
-		<li>
-		<note>
+		w<dd>
+		w<dt>
+		<e>
+		<i>
+		<ident>
+		w<impo>
+		<keyword>
+		w<li>
+		<mail>
+		w<note>
 		w<p>
+		<path>
 		W<pre>
-		<th>
-		<ti>
-		<title>
-		<warn>';
+		<stmt>
+		<sub>
+		w<subtitle>
+		w<summary>
+		<sup>
+		w<th>
+		w<ti>
+		w<title>
+		<uri>
+		<var>
+		<version>
+		w<warn>';
 	$self->{options}{'attributes'}.='
 		<author>title
 		<figure>caption
+		<figure>link
 		<figure>short
+		<guide>lang
 		<guide>link
+		<p>by
 		<pre>caption';
 	$self->{options}{'inline'}.='
+		<b>
+		<brite>
 		<c>
+		<const>
 		<e>
 		<i>
+		<ident>
+		<img>
+		<keyword>
+		<mail>
 		<path>
-		<uri>';
+		<stmt>
+		<sub>
+		<sup>
+		<uri>
+		<var>';
 	$self->treat_options;
 }
