@@ -226,6 +226,13 @@ not split the current paragraph. The string to translate will then contain
 I<foo E<lt>.bar baz quxE<gt> quux>, where I<bar> is the command that
 should be inlined, and I<baz qux> its arguments.
 
+=item B<unknown_macros>
+
+This option indicates how po4a should behave when an unknown macro is found.
+By default, po4a fails with a warning.
+It can take the following values: I<failed> (the default value),
+I<untranslated>, I<noarg>, I<translate_joined>, I<translate_each>.
+
 =back
 
 =head1 AUTHORING MAN PAGES COMPLIANT WITH PO4A::MAN
@@ -351,6 +358,7 @@ under the terms of GPL (see the COPYING file).
 =cut
 
 package Locale::Po4a::Man;
+use DynaLoader;
 
 use 5.006;
 use strict;
@@ -358,8 +366,12 @@ use warnings;
 
 require Exporter;
 use vars qw(@ISA @EXPORT);
-@ISA = qw(Locale::Po4a::TransTractor);
+@ISA = qw(Locale::Po4a::TransTractor DynaLoader);
 @EXPORT = qw();#  new initialize);
+
+# Try to use a C extension if present.
+eval('bootstrap Locale::Po4a::Man "0.30"');
+
 use Locale::Po4a::TransTractor;
 use Locale::Po4a::Common;
 
@@ -367,6 +379,7 @@ use File::Spec;
 use Getopt::Std;
 
 my %macro; # hash of known macro, with parsing sub. See end of this file
+my %default_macro; # The default known macros, when no options are used.
 
 # A font start by \f and is followed either by
 # [.*] - a font name within brackets (e.g. [P], [A_USER_FONT])
@@ -383,46 +396,43 @@ my $nbs;
 # Indicate if the page uses the mdoc macros
 my $mdoc_mode = 0;
 
+my $unknown_macros = undef;
+
 #########################
 #### DEBUGGING STUFF ####
 #########################
-my %debug=('splitargs' => 0, # see how macro args are separated
-	   'pretrans' => 0,  # see pre-conditioning of translation
-	   'postrans' => 0,  # see post-conditioning of translation
-	   'fonts'    => 0,  # see font modifier handling
-	   );
+my %debug;
+# The following debug options can be set with '-o debug=...':
+# * splitargs  see how macro args are separated
+# * pretrans   see pre-conditioning of translation
+# * postrans   see post-conditioning of translation
+# * fonts      see font modifier handling
 
 
 ######## CONFIG #########
 # This variable indicates the behavior of the module when a .de, .if or
 # .ie is encountered.
-my $groff_code = "fail";
+my $groff_code;
 # %no_wrap_begin and %no_wrap_end are lists of macros that respectively
 # begins and ends a no_wrap paragraph.
 # Any ending macro will end the no_wrap paragraph started by any beginning
 # macro.
-my %no_wrap_begin = (
-    'nf' => 1,
-    'EX' => 1,
-    'EQ' => 1
-);
-my %no_wrap_end = (
-    'fi' => 1,
-    'EE' => 1,
-    'EN' => 1
-);
-my %inline = (
-);
+my %no_wrap_begin;
+my %no_wrap_end;
+# List of macros that should be inlined (with E<.xx ...>)
+my %inline;
+# The default list of inlined macros (when no options are used)
+my %default_inline;
 # This variable indicates whether po4a should try to detect the generated
 # files.
-my $allow_generated = 0;
+my $allow_generated;
 # This hash indicates section name that should not be translated in mdoc
 # mode.
 # The groff's mdoc processor requires the NAME section, otherwise headers
 # and footers of the pages are not generated.
 # The mdoc_groff man page indicates that NAME, SYNOPSIS and DESCRIPTION
 # are mandatory.
-my %mdoc = ();
+my %mdoc;
 sub initialize {
     my $self = shift;
     my %options = @_;
@@ -438,9 +448,7 @@ sub initialize {
     $self->{options}{'inline'}='';
     $self->{options}{'generated'}='';
     $self->{options}{'mdoc'}='';
-
-    # reset the debug options
-    %debug = ();
+    $self->{options}{'unknown_macros'}='';
 
     foreach my $opt (keys %options) {
         if (defined $options{$opt}) {
@@ -451,12 +459,14 @@ sub initialize {
         }
     }
 
+    %debug = ();
     if (defined $options{'debug'}) {
         foreach ($options{'debug'}) {
             $debug{$_} = 1;
         }
     }
 
+    $groff_code = "fail";
     if (defined $options{'groff_code'}) {
         unless ($options{'groff_code'} =~ m/fail|verbatim|translate/) {
           die wrap_mod("po4a::man", dgettext("po4a",
@@ -465,6 +475,11 @@ sub initialize {
         $groff_code = $options{'groff_code'};
     }
 
+    if (%default_macro) {
+        %macro = %default_macro;
+    } else {
+        %default_macro = %macro
+    }
     if (defined $options{'untranslated'}) {
         foreach (split(/,/, $options{'untranslated'})) {
             $macro{$_} = \&untranslated;
@@ -485,6 +500,17 @@ sub initialize {
             $macro{$_} = \&translate_each;
         }
     }
+
+    %no_wrap_begin = (
+        'nf' => 1,
+        'EX' => 1,
+        'EQ' => 1
+    );
+    %no_wrap_end = (
+        'fi' => 1,
+        'EE' => 1,
+        'EN' => 1
+    );
     if (defined $options{'no_wrap'}) {
         foreach (split(/,/, $options{'no_wrap'})) {
             if ($_ =~ m/^(.*):(.*)$/) {
@@ -495,14 +521,24 @@ sub initialize {
             }
         }
     }
+
+    if (%default_inline) {
+        %inline = %default_inline;
+    } else {
+        %default_inline = %inline
+    }
     if (defined $options{'inline'}) {
         foreach (split(/,/, $options{'inline'})) {
             $inline{$_} = 1;
         }
     }
+
+    $allow_generated = 0;
     if (defined $options{'generated'}) {
         $allow_generated = 1;
     }
+
+    %mdoc = ();
     if (defined $options{'mdoc'}) {
         if ($options{'mdoc'} eq 1) {
             $mdoc{"NAME"} = 1;
@@ -512,6 +548,25 @@ sub initialize {
             }
         }
     }
+
+    $unknown_macros = undef;
+    if (defined $options{'unknown_macros'}) {
+        if ($options{'unknown_macros'} eq "failed") {
+            $unknown_macros = undef;
+        } elsif ($options{'unknown_macros'} eq "untranslated") {
+            $unknown_macros = \&untranslated;
+        } elsif ($options{'unknown_macros'} eq "noarg") {
+            $unknown_macros = \&noarg;
+        } elsif ($options{'unknown_macros'} eq "translate_joined") {
+            $unknown_macros = \&translate_joined;
+        } elsif ($options{'unknown_macros'} eq "translate_each") {
+            $unknown_macros = \&translate_each;
+        } else {
+            die wrap_mod("po4a::man", dgettext("po4a",
+                "Invalid 'unknown_macros' value. Must be one of:\n").
+                "failed untranslated noarg translate_joined translate_each\n");
+        }
+    }
 }
 
 my @comments = ();
@@ -743,7 +798,7 @@ sub pushline {
         # add comments
         foreach my $c (@comments) {
             # comments are pushed (maybe at the wrong place).
-            $self->SUPER::pushline(".\\\"$c\n");
+            $self->SUPER::pushline($self->r(".\\\"$c\n"));
         }
         @comments = ();
     }
@@ -967,12 +1022,8 @@ sub post_trans {
     $str =~ s/\n([.'])/ $1/mg;
 
     # Change ascii non-breaking space to groff one
-    my $nbs_out = "\xA0";
-    my $enc_length;
-    eval("\$enc_length = Encode::from_to(\$nbs_out, \"latin1\",
-                                         \$self->get_out_charset,
-                                         1)");
-    $str =~ s/\Q$nbs_out/\\ /sg if defined $enc_length;
+    my $nbs_out = get_out_nbs($self->get_out_charset);
+    $str =~ s/\Q$nbs_out/\\ /sg if defined $nbs_out;
     # No nbsp (said "\ " in groff on the last pos of the line, or groff adds
     # an extra space
     $str =~ s/\\ \n/\\ /sg;
@@ -1005,7 +1056,7 @@ sub post_trans {
         $t1 =~ s/ +$//s;
         $t2 =~ s/\n/ /gs;
         if ($mdoc_mode) {
-            # restore the punctuaction inside the line (see pre_trans)
+            # restore the punctuation inside the line (see pre_trans)
             if ($t3 =~ s/^([.,;:\)\]]+)//s) {
                 my $punctuation = $1;
                 $punctuation =~ s/([.,;:\)\]])/$1 /;
@@ -1078,7 +1129,7 @@ sub translate {
     # Do not translate the strings that only consist of fonts, spaces and
     # \&. This is useful because we introduced \& in shiftline.
     if ($str =~ m/^($FONT_RE|\s|\\&)*$/s) {
-        do_fonts($str, $ref);
+        do_fonts($str, $ref||$self->{ref});
         return $str;
     }
 
@@ -1116,6 +1167,13 @@ sub t {
     return $_[0]->translate($_[1]);
 }
 
+# shortcut.
+# As a rule of thumb, I do not recode macro names, unless they may be
+# followed by other characters.
+sub r {
+    return $_[0]->recode_skipped_text($_[1]);
+}
+
 
 sub do_paragraph {
     my ($self,$paragraph,$wrapped_mode) = (shift,shift,shift);
@@ -1147,19 +1205,9 @@ sub parse{
                             #          until the next fi macro.
 
 
-    # change the non-breaking space according to the input document charset
-    $nbs = "\xA0";
-    my $enc_length;
-    if (defined $self->{TT}{'file_in_charset'} and
-        length $self->{TT}{'file_in_charset'})
-    {
-        eval ("\$enc_length = Encode::from_to(\$nbs, \"latin-1\",
-                                              \$self->{TT}{'file_in_charset'},
-                                              1)");
-    }
-    # fall back solution
-    $nbs = "PO4A:VERY_IMPROBABLE_STRING_USEDFOR_NON-BREAKING-SPACES"
-        unless defined $enc_length;
+    # We want to change the non-breaking space according to the input
+    # document charset
+    $nbs = get_in_nbs($self->{TT}{'file_in_charset'});
 
   LINE:
     undef $self->{type};
@@ -1213,7 +1261,7 @@ sub parse{
 	    #       => substitution like Perl's tr/ac/bd/ on output.
 	    if ($macro eq '\\"' || $macro eq '' || $macro eq 'tr' ||
 	        $macro eq '"'   || $macro eq '\\#') {
-		$self->pushline($line."\n");
+		$self->pushline($self->r($line)."\n");
 		goto LINE;
 	    }
 	    # Special case:
@@ -1225,7 +1273,7 @@ sub parse{
 		} else {
 		    $wrapped_mode='MACRONO';
 		}
-		$self->pushline($line."\n");
+		$self->pushline($self->r($line)."\n");
 		goto LINE;
 	    }
 	    
@@ -1241,9 +1289,13 @@ sub parse{
 	    if (defined ($macro{$macro})) {
 		&{$macro{$macro}}(@args);
 	    } else {
-		$self->pushline($line."\n");
+		if (defined $unknown_macros) {
+		    &{$unknown_macros}(@args);
+		} else {
+		$self->pushline($self->r($line)."\n");
 		die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
 		    "Unknown macro '%s'. Remove it from the document, or refer to the Locale::Po4a::Man manpage to see how po4a can handle new macros."), $line);
+		}
 	    }
 
 	} elsif ($line =~ /^ +[^. ]/) {
@@ -1260,7 +1312,7 @@ sub parse{
 		# From info groff:
 		# Escape: \": Start a comment.  Everything to the end of the
 		# input line is ignored.
-		$self->pushline($line."\n");
+		$self->pushline($self->r($line)."\n");
 		goto LINE;
 	    } elsif ($line =~ /^\\#/) {
 		# Special groff comment. Do not keep the new line
@@ -1302,7 +1354,7 @@ sub parse{
     @next_comments = @comments;
     @comments = ();
     for my $c (@next_comments) {
-	$self->pushline(".\\\"$c\n");
+	$self->pushline($self->r(".\\\"$c\n"));
     }
 
     # reinitialize the module
@@ -1313,6 +1365,74 @@ sub parse{
     $mdoc_mode = 0;
 } # end of main
 
+# Cache the results of get_in_nbs and get_out_nbs
+{
+    my $last_in_charset;
+    my $last_in_nbs;
+
+# get_in_nbs(charset)
+# Return the representation of a non breaking space in the input charset
+# (given in argument).
+# or PO4A:VERY_IMPROBABLE_STRING_USEDFOR_NON-BREAKING-SPACES if this
+# character doesn't exist in this charset.
+    sub get_in_nbs() {
+        my $charset = shift;
+
+        return $last_in_nbs
+            if (    defined $charset
+                and defined $last_in_charset
+                and $charset eq $last_in_charset);
+
+        my $nbs = "\xA0";
+        my $length;
+        if (defined $charset and length $charset)
+        {
+            eval ("\$length = Encode::from_to(\$nbs, \"latin-1\",
+                                              \$charset,
+                                              1)");
+        }
+        # fall back solution
+        $nbs = "PO4A:VERY_IMPROBABLE_STRING_USEDFOR_NON-BREAKING-SPACES"
+            unless defined $length;
+        $last_in_charset = $charset;
+        $last_in_nbs = $nbs;
+
+        return $last_in_nbs;
+    }
+
+    my $last_out_charset;
+    my $last_out_nbs;
+# get_out_nbs(charset)
+# Return the representation of a non breaking space in the output charset
+# (given in argument).
+# or undef if this character doesn't exist in this charset.
+    sub get_out_nbs() {
+        my $charset = shift;
+
+        return $last_out_nbs
+            if (    defined $charset
+                and defined $last_out_charset
+                and $charset eq $last_out_charset);
+
+        my $nbs = "\xA0";
+        my $length;
+        if (defined $charset and length $charset)
+        {
+            eval ("\$length = Encode::from_to(\$nbs, \"latin-1\",
+                                              \$charset,
+                                              1)");
+        }
+        # fall back solution
+        undef $nbs
+            unless defined $length;
+        $last_out_charset = $charset;
+        $last_out_nbs = $nbs;
+
+        return $last_out_nbs;
+    }
+
+}
+
 # We can't push the header in the first line of the document, as in the
 # other module, because the first line may contain indications on how the
 # man page must be processed.
@@ -1579,7 +1699,7 @@ sub splitargs {
             $str =~ s/($FONT_RE)($FONT_RE)/$2/s;
         }
 
-        # when there are two concecutive switches to the regular font,
+        # when there are two consecutive switches to the regular font,
         # remove the last one.
         while ($str =~ /^(.*)\\f$regular_font # anything followed by a
                                               # regular font
@@ -1657,7 +1777,7 @@ sub translate_joined {
 		     $self->t($macroarg));
 }
 
-# For macro taking several arguments, having to be translated separatly
+# For macro taking several arguments, having to be translated separately
 sub translate_each {
     my ($self,$first)= (shift,0);
     $self->pushmacro( map { $first++ ?$self->t($_):$_ } @_);
@@ -1671,7 +1791,7 @@ sub noarg {
     $self->pushmacro(@_);
 }
 
-# For macro whose arguments shouln't be translated
+# For macro whose arguments shouldn't be translated
 sub untranslated {
     my $self = shift;
     $self->pushmacro(@_);
@@ -1755,12 +1875,12 @@ $macro{'TP'}=sub {
     my ($line,$l2,$ref2);
     $line .= $_[0] if defined($_[0]);
     $line .= ' '.$_[1] if defined($_[1]);
-    $self->pushline($line."\n");
+    $self->pushline($self->r($line)."\n");
 
     ($l2,$ref2) = $self->shiftline();
     chomp($l2);
     while ($l2 =~ /^\.PD/) {
-	$self->pushline($l2."\n");
+	$self->pushline($self->r($l2)."\n");
 	($l2,$ref2) = $self->shiftline();
 	chomp($l2);
     }
@@ -1900,7 +2020,7 @@ $macro{'de'}=sub {
         }
         $paragraph .= "\n";
         if ($groff_code eq "verbatim") {
-            $self->pushline($paragraph);
+            $self->pushline( $self->r($paragraph) );
         } else {
             $self->pushline( $self->translate($paragraph,
                                               $self->{ref},
@@ -1921,7 +2041,7 @@ $macro{'ds'}=sub {
     # find references to this string in the translation "\*(name" or
     # "\*[name]"
     $self->{type} = "ds $name";
-    $self->pushline($m." ".$name." ".$self->translate($string)."\n");
+    $self->pushline($m." ".$self->r($name)." ".$self->translate($string)."\n");
 };
 #       .fam      Return to previous font family.
 #       .fam name Set the current font family to name.
@@ -1991,7 +2111,7 @@ $macro{'ie'}=$macro{'if'}=sub {
         }
         $paragraph .= "\n";
         if ($groff_code eq "verbatim") {
-            $self->pushline($paragraph);
+            $self->pushline( $self->r($paragraph) );
         } else {
             $self->pushline( $self->translate($paragraph,
                                               $self->{ref},
@@ -2014,7 +2134,7 @@ $macro{'ig'}=sub {
     $end='' if ($end =~ m/^\\\"/);
     my ($line,$ref)=$self->shiftline();
     while (defined($line)) {
-	$self->pushline($line);
+	$self->pushline($self->r($line));
 	last if ($line =~ /^\.$end\./);
 	($line,$ref)=$self->shiftline();
     }
@@ -2083,14 +2203,14 @@ $macro{'TS'}=sub {
     while (defined($line)) {
 	if ($line =~ /^\.TE/) {
 	    # Table end
-	    $self->pushline($line);
+	    $self->pushline($self->r($line));
 	    return;
 	}
 	if ($in_headers) {
 	    if ($line =~ /\.$/) {
 		$in_headers = 0;
 	    }
-	    $self->pushline($line);
+	    $self->pushline($self->r($line));
 	} elsif ($line =~ /\\$/) {
 	    # Lines are continued on \ at the end of line
 	    $buffer .= $line;
@@ -2120,7 +2240,7 @@ $macro{'F'}=$macro{'H'}=$macro{'V'}=$macro{'A'}=$macro{'T'}=\&untranslated;
 ##
 #
 # Displays and keeps. None of these macro accept a translated argument
-# (they allow to make blocks of text which cannot be breaked by new page)
+# (they allow to make blocks of text which cannot be broken by new page)
 
 $macro{'DS'}=$macro{'LD'}=$macro{'DE'}=\&untranslated;
 $macro{'ID'}=$macro{'BD'}=$macro{'CD'}=\&untranslated;
@@ -2143,7 +2263,7 @@ $macro{'cs'}=\&untranslated;
 # Change the minimum size of a space between filled words.
 $macro{'ss'}=\&untranslated;
 
-# .ce     Center one line horizontaly
+# .ce     Center one line horizontally
 # .ce N   Center N lines
 # .ul N   Underline N lines (but not the spaces)
 # .cu N   Underline N lines (even the spaces)
@@ -2192,7 +2312,7 @@ $macro{'UC'}=$macro{'AT'}=\&untranslated;
 #   Define how WORD1, WORD2, etc. are to be hyphenated.  The words
 #   must be given with hyphens at the hyphenation points.
 #
-#   If the english page needs to specify how a word must be hyphanted, the
+#   If the English page needs to specify how a word must be hyphenated, the
 #   translated page may also have this need.
 $macro{'hw'}=\&translate_each;
 
@@ -2213,7 +2333,7 @@ sub translate_mdoc {
     my $macroarg = "";
     foreach (@_) {
         $macroarg.=" " if (length $macroarg);
-        if ($_ =~ m/((?<!\\) |\t)/) {
+        if ($_ =~ m/((?<!\\) |\t|^$)/) {
             $macroarg.="\"$_\"";
         } else {
             $macroarg.=$_;
@@ -2272,14 +2392,14 @@ sub define_mdoc_macros {
         my $macroarg = "";
         foreach (@_) {
             $macroarg.=" " if (length $macroarg);
-            if ($_ =~ m/((?<!\\) |\t)/) {
+            if ($_ =~ m/((?<!\\) |\t|^$)/) {
                 $macroarg.="\"$_\"";
             } else {
                 $macroarg.=$_;
             }
         }
         if ($mdoc{$macroarg}) {
-            $self->pushline("$macroname ".$macroarg."\n");
+            $self->pushline("$macroname ".$self->r($macroarg)."\n");
         } else {
             $self->pushline("$macroname ".$self->t($macroarg)."\n");
         }
