@@ -1,5 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files 
-# $Id: Po.pm,v 1.65 2006/10/07 20:33:28 nekral-guest Exp $
+# $Id: Po.pm,v 1.76 2007/04/18 08:36:04 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -66,28 +66,34 @@ use IO::File;
 require Exporter;
 
 package Locale::Po4a::Po;
+use DynaLoader;
+
+use Locale::Po4a::Common qw(wrap_msg wrap_mod wrap_ref_mod dgettext);
+
+use subs qw(makespace);
+use vars qw(@ISA @EXPORT_OK);
+@ISA = qw(Exporter DynaLoader);
+@EXPORT = qw(%debug);
+@EXPORT_OK = qw(&move_po_if_needed);
 
 use Locale::Po4a::TransTractor;
-use Locale::Po4a::Common qw(wrap_msg wrap_mod wrap_ref_mod dgettext);
+# Try to use a C extension if present.
+eval("bootstrap Locale::Po4a::Po $Locale::Po4a::TransTractor::VERSION");
 
 use 5.006;
 use strict;
 use warnings;
 
-use subs qw(makespace);
-use vars qw(@ISA @EXPORT);
-@ISA = ();
-@EXPORT = qw(read write write_if_needed gettext);
-
 use Carp qw(croak);
 use File::Path; # mkdir before write
 use File::Copy; # move
+use POSIX qw(strftime floor);
 
 use Encode;
 
 my @known_flags=qw(wrap no-wrap c-format fuzzy);
 
-my %debug=('canonize'	=> 0,
+our %debug=('canonize'	=> 0,
            'quote'	=> 0,
            'escape'	=> 0,
            'encoding'   => 0,
@@ -116,13 +122,32 @@ sub new {
     return $self;
 }
 
+# Return the numerical timezone (e.g. +0200)
+# The %z format of strftime is not portable.
+# For example, it indicates "2006-10-25 19:36E. Europe Standard Time" on
+# MS Windows.
+sub timezone {
+    my @g = gmtime();
+    my @l = localtime();
+
+    my $diff = floor(strftime("%s", @l)/60 +0.5) - floor(strftime("%s", @g)/60 + 0.5);
+
+    my $h = floor($diff / 60) + $l[8]; # $l[8] indicates if we are currently
+                                       # in a daylight saving time zone
+    my $m = $diff%60;
+
+    return sprintf "%+03d%02d\n", $h, $m;
+}
+
 sub initialize {
     my ($self, $options) = (shift, shift);
-    my $date = `date +'%Y-%m-%d %k:%M%z'`;
+    my $date = strftime("%Y-%m-%d %H:%M", localtime).timezone();
     chomp $date;
 #    $options = ref($options) || $options;
 
     $self->{options}{'porefs'}= 'full';
+    $self->{options}{'msgid-bugs-address'}= undef;
+    $self->{options}{'copyright-holder'}= "Free Software Foundation, Inc.";
     foreach my $opt (keys %$options) {
 	if ($options->{$opt}) {
 	    die wrap_mod("po4a::po", dgettext ("po4a", "Unknown option: %s"), $opt) unless exists $self->{options}{$opt};
@@ -141,12 +166,15 @@ sub initialize {
     $self->{count_doc}=0;
     $self->{header_comment}=
 	escape_text( " SOME DESCRIPTIVE TITLE\n"
-		    ." Copyright (C) YEAR Free Software Foundation, Inc.\n"
+		    ." Copyright (C) YEAR ".$self->{options}{'copyright-holder'}."\n"
 		    ." FIRST AUTHOR <EMAIL\@ADDRESS>, YEAR.\n"
 		    ." \n"
 		    .", fuzzy");
 #    $self->header_tag="fuzzy";
     $self->{header}=escape_text("Project-Id-Version: PACKAGE VERSION\n".
+			((defined $self->{options}{'msgid-bugs-address'})?
+	"Report-Msgid-Bugs-To: ".$self->{options}{'msgid-bugs-address'}."\n":
+				"").
 				"POT-Creation-Date: $date\n".
 				"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n".
 				"Last-Translator: FULL NAME <EMAIL\@ADDRESS>\n".
@@ -155,6 +183,8 @@ sub initialize {
 				"Content-Type: text/plain; charset=CHARSET\n".
 				"Content-Transfer-Encoding: ENCODING");
 
+    $self->{encoder}=find_encoding("ascii");
+
     # To make stats about gettext hits
     $self->stats_clear();
 }
@@ -167,7 +197,7 @@ catalog.
 
 =cut
 
-sub read{
+sub read {
     my $self=shift;
     my $filename=shift 
 	|| croak wrap_mod("po4a::po", dgettext("po4a", "Please provide a non-null filename"));
@@ -325,27 +355,33 @@ update a line reference or the POT-Creation-Date field).
 =cut
 
 sub move_po_if_needed {
-    my ($new_po, $old_po) = (shift, shift);
+    my ($new_po, $old_po, $backup) = (shift, shift, shift);
     my $diff;
 
     if (-e $old_po) {
         $diff = qx(diff -q -I'^#:' -I'^"POT-Creation-Date:' -I'^"PO-Revision-Date:' $old_po $new_po);
         if ( $diff eq "" ) {
             unlink $new_po
-                or die wrap_msg(dgettext("po4a","Can't unlink %s."),
-                                $new_po);
+                or die wrap_msg(dgettext("po4a","Can't unlink %s: %s."),
+                                $new_po, $!);
             # touch the old PO
             my ($atime, $mtime) = (time,time);
             utime $atime, $mtime, $old_po;
         } else {
+            if ($backup) {
+                copy $old_po, $old_po."~"
+                  or die wrap_msg(dgettext("po4a","Can't copy %s to %s: %s."),
+                                  $old_po, $old_po."~", $!);
+            } else {
+            }
             move $new_po, $old_po
-                or die wrap_msg(dgettext("po4a","Can't move %s to %s."),
-                                $new_po, $old_po);
+                or die wrap_msg(dgettext("po4a","Can't move %s to %s: %s."),
+                                $new_po, $old_po, $!);
         }
     } else {
         move $new_po, $old_po
-            or die wrap_msg(dgettext("po4a","Can't move %s to %s."),
-                            $new_po, $old_po);
+            or die wrap_msg(dgettext("po4a","Can't move %s to %s: %s."),
+                            $new_po, $old_po, $!);
     }
 }
 
@@ -968,6 +1004,12 @@ sub push_raw {
 #	} FIXME: do that iff the header isn't the default one.
 	$self->{header}=$msgstr;
 	$self->{header_comment}=$comment;
+	my $charset = $self->get_charset;
+	if ($charset ne "CHARSET") {
+	    $self->{encoder}=find_encoding($charset);
+	} else {
+	    $self->{encoder}=find_encoding("ascii");
+	}
 	return;
     }
 
@@ -1135,6 +1177,7 @@ sub set_charset() {
     $oldchar = $self->get_charset();
 
     $self->{header} =~ s/$oldchar/$newchar/;
+    $self->{encoder}=find_encoding($newchar);
 }
 
 #----[ helper functions ]---------------------------------------------------
