@@ -3,6 +3,7 @@
 require Exporter;
 
 package Locale::Po4a::TransTractor;
+use DynaLoader;
 
 use 5.006;
 use strict;
@@ -10,12 +11,15 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.29";
-@ISA = ();
-@EXPORT = qw(process translate 
-	     read write readpo writepo
+$VERSION="0.31";
+@ISA = qw(DynaLoader);
+@EXPORT = qw(new process translate 
+             read write readpo writepo
              getpoout setpoout);
 
+# Try to use a C extension if present.
+eval("bootstrap Locale::Po4a::TransTractor $VERSION");
+
 use Carp qw(croak);
 use Locale::Po4a::Po;
 use Locale::Po4a::Common;
@@ -280,8 +284,12 @@ sub process {
 	$newparams{$_}=$params{$_};
     }
 
-    $self->{TT}{'file_in_charset'}=$params{'file_in_charset'};
+    $self->detected_charset($params{'file_in_charset'});
     $self->{TT}{'file_out_charset'}=$params{'file_out_charset'};
+    if (defined($self->{TT}{'file_out_charset'}) and
+	length($self->{TT}{'file_out_charset'})) {
+	$self->{TT}{'file_out_encoder'} = find_encoding($self->{TT}{'file_out_charset'});
+    }
     $self->{TT}{'addendum_charset'}=$params{'addendum_charset'};
 
     foreach my $file (@{$params{'po_in_name'}}) {
@@ -392,14 +400,16 @@ sub read() {
 	my @entry=($textline,$ref);
 	push @{$self->{TT}{doc_in}}, @entry;
 
-	# Detect if this file has non-ascii characters
-	if($self->{TT}{ascii_input}) {
-	    my $decoder = guess_encoding($textline);
-	    if (!ref($decoder) or $decoder !~ /Encode::XS=/) {
-	        # We have detected a non-ascii line
-		$self->{TT}{ascii_input} = 0;
-		# Save the reference for future error message
-		$self->{TT}{non_ascii_ref} ||= $ref;
+	if (!defined($self->{TT}{'file_in_charset'})) {
+	    # Detect if this file has non-ascii characters
+	    if($self->{TT}{ascii_input}) {
+		my $decoder = guess_encoding($textline);
+		if (!ref($decoder) or $decoder !~ /Encode::XS=/) {
+		    # We have detected a non-ascii line
+		    $self->{TT}{ascii_input} = 0;
+		    # Save the reference for future error message
+		    $self->{TT}{non_ascii_ref} ||= $ref;
+		}
 	    }
 	}
     }
@@ -620,7 +630,7 @@ sub addendum {
     }
     if ($found > 1) {
 	warn wrap_msg(dgettext("po4a",
-	    "More than one cadidate position found for the addendum %s."), $filename);
+	    "More than one candidate position found for the addendum %s."), $filename);
 	return 0;
     }
 
@@ -818,8 +828,7 @@ sub translate {
     } else {
 	if (defined($self->{TT}{'file_in_charset'}) and
 	    length($self->{TT}{'file_in_charset'}) and
-	    $self->{TT}{'file_in_charset'} ne "CHARSET" and
-	    $self->{TT}{'file_in_charset'} ne "ascii") {
+	    $self->{TT}{'file_in_charset'} !~ m/ascii/i) {
 	    $in_charset=$self->{TT}{'file_in_charset'};
 	} else {
 	    # FYI, the document charset have to be determined *before* we see the first
@@ -829,7 +838,9 @@ sub translate {
     }
 
     if ($self->{TT}{po_in}->get_charset ne "CHARSET") {
-	Encode::from_to($string, $in_charset, $self->{TT}{po_in}->get_charset);
+	$string = encode_from_to($string,
+	                         $self->{TT}{'file_in_encoder'},
+	                         $self->{TT}{po_in}{encoder});
     }
 
     if (defined $options{'wrapcol'} && $options{'wrapcol'} < 0) {
@@ -841,8 +852,13 @@ sub translate {
 					'wrapcol'   => $options{'wrapcol'});
 
     if ($self->{TT}{po_in}->get_charset ne "CHARSET") {
-	Encode::from_to($transstring,$self->{TT}{po_in}->get_charset,
-	    $self->get_out_charset);
+	my $out_encoder = $self->{TT}{'file_out_encoder'};
+	unless (defined $out_encoder) {
+	    $out_encoder = find_encoding($self->get_out_charset)
+	}
+	$transstring = encode_from_to($transstring,
+	                              $self->{TT}{po_in}{encoder},
+	                              $out_encoder);
     }
 
     # If the input document isn't completely in ascii, we should see what to
@@ -923,9 +939,17 @@ process() arguments or detected from the document.
 sub detected_charset {
     my ($self,$charset)=(shift,shift);
     unless (defined($self->{TT}{'file_in_charset'}) and
-	length($self->{TT}{'file_in_charset'}) ) {
+            length($self->{TT}{'file_in_charset'}) ) {
+        $self->{TT}{'file_in_charset'}=$charset;
+        if (defined $charset) {
+            $self->{TT}{'file_in_encoder'}=find_encoding($charset);
+        }
+    }
 
-	$self->{TT}{'file_in_charset'}=$charset;
+    if (defined $self->{TT}{'file_in_charset'} and
+        length $self->{TT}{'file_in_charset'} and
+        $self->{TT}{'file_in_charset'} !~ m/ascii/i) {
+	$self->{TT}{ascii_input}=0;
     }
 }
 
@@ -981,8 +1005,9 @@ sub recode_skipped_text {
     unless ($self->{TT}{'ascii_input'}) {
 	if(defined($self->{TT}{'file_in_charset'}) and
 	    length($self->{TT}{'file_in_charset'}) ) {
-	    Encode::from_to($text,$self->{TT}{'file_in_charset'},
-		$self->get_out_charset);
+	    $text = encode_from_to($text,
+	                           $self->{TT}{'file_in_encoder'},
+	                           find_encoding($self->get_out_charset));
 	} else {
 	    die wrap_mod("po4a", dgettext("po4a", "Couldn't determine the input document's charset. Please specify it on the command line. (non-ascii char at %s)"), $self->{TT}{non_ascii_ref})
 	}
@@ -990,6 +1015,38 @@ sub recode_skipped_text {
     return $text;
 }
 
+
+# encode_from_to($,$,$)
+#
+# Encode the given text from one encoding to another one.
+# It differs from Encode::from_to because it does not take the name of the
+# encoding in argument, but the encoders (as returned by the
+# Encode::find_encoding(<name>) method). Thus it permits to save a bunch
+# of call to find_encoding.
+#
+# If the "from" encoding is undefined, it is considered as UTF-8 (or
+# ascii).
+# If the "to" encoding is undefined, it is considered as UTF-8.
+#
+sub encode_from_to {
+    my ($text,$from,$to) = (shift,shift,shift);
+
+    if (not defined $from) {
+        # for ascii and UTF-8, no conversion needed to get an utf-8
+        # string.
+    } else {
+        $text = $from->decode($text, 0);
+    }
+
+    if (not defined $to) {
+        # Already in UTF-8, no conversion needed
+    } else {
+        $text = $to->encode($text, 0);
+    }
+
+    return $text;
+}
+
 =back
 
 =head1 FUTURE DIRECTIONS
